/* ================================================================================
 *
 * DMASDSPI API.
 *
 * Copyright (c) 2016, KIICHIRO KOTAJIMA
 * All Rights Reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ================================================================================
*/
//#include "project.h"
#include "cyfitter_sysint_cfg.h"
#include "DMASDSPI_api.h"
#include "DMASDSPI_fatfs.h"

#include "DMASDSPI_SPIMaster.h"
#include "DMASDSPI_CSControl.h"
//#include "DMASDSPI_CardStatus.h"
#include "DMASDSPI_RX.h"
#include "DMASDSPI_TX.h"
//#include "DMASDSPI_RX_DMA_IRQ.h"



#define TRANSFER_WAIT_RXDMAISR     1

static volatile uint8_t nTransferFlags = 0;

static uint8_t bEnableDMA = 0;

static uint8_t dummy_data[1] = { 0xff };

#ifdef DMASDSPI_RX_DMA_IRQ__INTC_CORTEXM0P_PRIORITY

//CM0+ interrupt code

#endif
#ifdef DMASDSPI_RX_DMA_IRQ__INTC_CORTEXM4_PRIORITY
//CM4 interrupt code
void DMASDSPI_RX_DMA_ISR(){
	nTransferFlags &= ~TRANSFER_WAIT_RXDMAISR;
}
#endif



void DMASDSPI_Start(DMASDSPI_VOIDFXN FnSelSlowCk, DMASDSPI_VOIDFXN FnSelFastCk, DMASDSPI_PRNTFXN FnPrint)
{
	nTransferFlags = 0;
	DMASDSPI_SPIMaster_Start();
	DMASDSPI_SetDMAMode(0);
	
	SPI_FUNCTIONS SpiFuncs;
	SpiFuncs.SPRXSTREAM = DMASDSPI_ReceiveData;
	SpiFuncs.SPTXSTREAM = DMASDSPI_SendData;
	SpiFuncs.SPEXCHANGE = DMASDSPI_SwapByte;
	SpiFuncs.SPSELECTCARD = DMASDSPI_SelectCard;
	SpiFuncs.SPDESELECTCARD = DMASDSPI_DeselectCard;
	SpiFuncs.SPFASTCK = FnSelFastCk;
	SpiFuncs.SPSLOWCK = FnSelSlowCk;
	SpiFuncs.CARDSTATUS = DMASDSPI_CardStatus;
	SpiFuncs.PRINT = FnPrint;
	
	disk_attach_spifuncs(&SpiFuncs);
}

void DMASDSPI_Stop()
{
	DMASDSPI_DeselectCard();
	DMASDSPI_SetDMAMode(0);
	DMASDSPI_SPIMaster_Disable();
}

void DMASDSPI_SelectCard() {
	DMASDSPI_CSControl_Write(0);
}

void DMASDSPI_DeselectCard() {
	DMASDSPI_CSControl_Write(1);
}

uint8_t DMASDSPI_CardStatus() {
    return 0x02;
}

void DMASDSPI_SendData(const uint8_t* pBuffer, uint32_t nLength)
{
	cy_stc_dma_channel_config_t channelConfig;
    if((nLength > 4095)||(nLength < 1)){
		return; //this never happens - filesys transfers max. 512 bytes at a time
	}

	if(bEnableDMA) {
                
        //TX
        Cy_DMA_Descriptor_Init(&DMASDSPI_TX_Descriptor_1, &DMASDSPI_TX_Descriptor_1_config);
        Cy_DMA_Descriptor_SetSrcAddress(&DMASDSPI_TX_Descriptor_1, pBuffer);
        Cy_DMA_Descriptor_SetDstAddress(&DMASDSPI_TX_Descriptor_1, DMASDSPI_SPIMaster_HW);

        channelConfig.descriptor    = &DMASDSPI_TX_Descriptor_1;
        channelConfig.preemptable   = DMASDSPI_TX_PREEMPTABLE;
        channelConfig.priority      = DMASDSPI_TX_PRIORITY;
        channelConfig.enable        = 0u;

        Cy_DMA_Channel_Init(DMASDSPI_TX_DW__BLOCK_HW, DMASDSPI_TX_DW_CHANNEL, &channelConfig);
        Cy_DMA_Channel_SetInterruptMask(DMASDSPI_TX_DW__BLOCK_HW, DMASDSPI_TX_DW__CHANNEL_NUMBER, CY_DMA_INTR_MASK);
        Cy_DMA_Channel_Enable(DMASDSPI_TX_DW__BLOCK_HW, DMASDSPI_TX_DW__CHANNEL_NUMBER);
        
        
        //RX
        Cy_DMA_Descriptor_Init(&DMASDSPI_RX_Descriptor_1, &DMASDSPI_RX_Descriptor_1_config);
        Cy_DMA_Descriptor_SetSrcAddress(&DMASDSPI_RX_Descriptor_1, DMASDSPI_SPIMaster_HW);
        Cy_DMA_Descriptor_SetDstAddress(&DMASDSPI_RX_Descriptor_1, &dummy_data[0]);

        channelConfig.descriptor    = &DMASDSPI_RX_Descriptor_1;
        channelConfig.preemptable   = DMASDSPI_RX_PREEMPTABLE;
        channelConfig.priority      = DMASDSPI_RX_PRIORITY;
        channelConfig.enable        = 0u;

        Cy_DMA_Channel_Init(DMASDSPI_RX_DW__BLOCK_HW, DMASDSPI_RX_DW_CHANNEL, &channelConfig);
        Cy_DMA_Channel_SetInterruptMask(DMASDSPI_RX_DW__BLOCK_HW, DMASDSPI_RX_DW__CHANNEL_NUMBER, CY_DMA_INTR_MASK);
        Cy_DMA_Channel_Enable(DMASDSPI_RX_DW__BLOCK_HW, DMASDSPI_RX_DW__CHANNEL_NUMBER);
        
        Cy_DMA_Channel_ClearInterrupt(DMASDSPI_TX_DW__BLOCK_HW, DMASDSPI_TX_DW__CHANNEL_NUMBER);
        Cy_DMA_Channel_ClearInterrupt(DMASDSPI_RX_DW__BLOCK_HW, DMASDSPI_RX_DW__CHANNEL_NUMBER);   
        	
		nTransferFlags |= TRANSFER_WAIT_RXDMAISR;   //Set flag
		
		Cy_DMA_Enable(DMASDSPI_RX_DW__BLOCK_HW); //CyDmaChEnable(RXDMA_Channel, 1);
		Cy_DMA_Enable(DMASDSPI_TX_DW__BLOCK_HW); //CyDmaChEnable(TXDMA_Channel, 1);
		
		//CyDmaChSetRequest(TXDMA_Channel, CPU_REQ);  //trigger transfer

		// wait for transfer
		while(nTransferFlags & TRANSFER_WAIT_RXDMAISR);

		// stop DMA.
        Cy_DMA_Disable(DMASDSPI_TX_DW__BLOCK_HW);
        Cy_DMA_Disable(DMASDSPI_RX_DW__BLOCK_HW);
		return;
	}else{
		//Slow (non DMA) implementation...
		unsigned int i;
		for(i = 0; i < nLength; i++){
			DMASDSPI_SwapByte(pBuffer[i]);
		}
	}
}

//Send a block of NULL data (default 0xFF) and place received bytes into buffer.
void DMASDSPI_ReceiveData(uint8_t* pBuffer, uint32_t nLength)
{
	cy_stc_dma_channel_config_t channelConfig;
    if((nLength > 4095)||(nLength < 1)){
		return; //this never happens - filesys transfers max. 512 bytes at a time
	}

	if(bEnableDMA) {
		//  - single TD
		//  - dummy_data() => SPI_TXDATA       
        //TX
        Cy_DMA_Descriptor_Init(&DMASDSPI_TX_Descriptor_1, &DMASDSPI_TX_Descriptor_1_config);
        Cy_DMA_Descriptor_SetSrcAddress(&DMASDSPI_TX_Descriptor_1, &dummy_data[0]);
        Cy_DMA_Descriptor_SetDstAddress(&DMASDSPI_TX_Descriptor_1, DMASDSPI_SPIMaster_HW);
        
        channelConfig.descriptor    = &DMASDSPI_TX_Descriptor_1;
        channelConfig.preemptable   = DMASDSPI_TX_PREEMPTABLE;
        channelConfig.priority      = DMASDSPI_TX_PRIORITY;
        channelConfig.enable        = 0u;
        
        Cy_DMA_Channel_Init(DMASDSPI_TX_DW__BLOCK_HW, DMASDSPI_TX_DW_CHANNEL, &channelConfig);
        Cy_DMA_Channel_SetInterruptMask(DMASDSPI_TX_DW__BLOCK_HW, DMASDSPI_TX_DW__CHANNEL_NUMBER, CY_DMA_INTR_MASK);
        Cy_DMA_Channel_Enable(DMASDSPI_TX_DW__BLOCK_HW, DMASDSPI_TX_DW__CHANNEL_NUMBER);
		
		//  - single TD
		//  - SPI_RXDATA => pBuffer()
        //RX      
        Cy_DMA_Descriptor_Init(&DMASDSPI_RX_Descriptor_1, &DMASDSPI_RX_Descriptor_1_config);
        Cy_DMA_Descriptor_SetSrcAddress(&DMASDSPI_RX_Descriptor_1, DMASDSPI_SPIMaster_HW);
        Cy_DMA_Descriptor_SetDstAddress(&DMASDSPI_RX_Descriptor_1, pBuffer);
        
        channelConfig.descriptor    = &DMASDSPI_RX_Descriptor_1;
        channelConfig.preemptable   = DMASDSPI_RX_PREEMPTABLE;
        channelConfig.priority      = DMASDSPI_RX_PRIORITY;
        channelConfig.enable        = 0u;
        
        Cy_DMA_Channel_Init(DMASDSPI_RX_DW__BLOCK_HW, DMASDSPI_RX_DW_CHANNEL, &channelConfig);
        Cy_DMA_Channel_SetInterruptMask(DMASDSPI_RX_DW__BLOCK_HW, DMASDSPI_RX_DW__CHANNEL_NUMBER, CY_DMA_INTR_MASK);
        Cy_DMA_Channel_Enable(DMASDSPI_RX_DW__BLOCK_HW, DMASDSPI_RX_DW__CHANNEL_NUMBER);
        
		dummy_data[0] = 0xff;    
		nTransferFlags |= TRANSFER_WAIT_RXDMAISR;   //Set flag
    
        Cy_DMA_Channel_ClearInterrupt(DMASDSPI_TX_DW__BLOCK_HW, DMASDSPI_TX_DW__CHANNEL_NUMBER);
        Cy_DMA_Channel_ClearInterrupt(DMASDSPI_RX_DW__BLOCK_HW, DMASDSPI_RX_DW__CHANNEL_NUMBER);  

		Cy_DMA_Enable(DMASDSPI_RX_DW__BLOCK_HW);
		Cy_DMA_Enable(DMASDSPI_TX_DW__BLOCK_HW);

		//CyDmaChSetRequest(TXDMA_Channel, CPU_REQ);  //trigger transfer

		//wait until done.
		while(nTransferFlags & TRANSFER_WAIT_RXDMAISR);

		// stop DMA.
        Cy_DMA_Disable(DMASDSPI_TX_DW__BLOCK_HW);
        Cy_DMA_Disable(DMASDSPI_RX_DW__BLOCK_HW);
		return;
	} else {
		//Slow (non DMA) implementation...
		unsigned int i;
		for(i = 0; i < nLength; i++){
			pBuffer[i] = DMASDSPI_SwapByte(0xFF);
		}
	}
}

//Swap a single byte (send it, and receive RX byte / without DMA)
uint8_t DMASDSPI_SwapByte(uint8_t data)
{
	//write byte to TX
	DMASDSPI_SPIMaster_Write(data);
	//wait for RX interrupt
	while(!(DMASDSPI_SPIMaster_GetRxFifoStatus()));
	//read byte from RX
	return DMASDSPI_SPIMaster_Read();
}

void DMASDSPI_SetDMAMode(unsigned char enable)
{
	if ((bEnableDMA && enable) || (!bEnableDMA && !enable))
		return;

	//make absolutely sure no transfers are in progress
	while(nTransferFlags);

	if (enable) {

        Cy_SysInt_Init(&DMASDSPI_RX_DMA_IRQ_cfg, DMASDSPI_RX_DMA_ISR);
        NVIC_EnableIRQ(DMASDSPI_RX_DMA_IRQ_cfg.intrSrc);

        Cy_DMA_Channel_ClearInterrupt(DMASDSPI_TX_DW__BLOCK_HW, DMASDSPI_TX_DW__CHANNEL_NUMBER);
        Cy_DMA_Channel_ClearInterrupt(DMASDSPI_RX_DW__BLOCK_HW, DMASDSPI_RX_DW__CHANNEL_NUMBER); 

		bEnableDMA = 1;
	} else {
		//Free Transfer
        NVIC_DisableIRQ(DMASDSPI_RX_DMA_IRQ_cfg.intrSrc);
		DMASDSPI_RX_ChannelDisable();
		DMASDSPI_TX_ChannelDisable();

		bEnableDMA = 0;
	}
}

/* [] END OF FILE */
