/*******************************************************************************
* File Name: SDSPI_CSControl_PM.c
* Version 1.80
*
* Description:
*  This file contains the setup, control, and status commands to support 
*  the component operation in the low power mode. 
*
* Note:
*
********************************************************************************
* Copyright 2015, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions, 
* disclaimers, and limitations in the end user license agreement accompanying 
* the software package with which this file was provided.
*******************************************************************************/

#include "SDSPI_CSControl.h"

/* Check for removal by optimization */
#if !defined(SDSPI_CSControl_Sync_ctrl_reg__REMOVED)

static SDSPI_CSControl_BACKUP_STRUCT  SDSPI_CSControl_backup = {0u};

    
/*******************************************************************************
* Function Name: SDSPI_CSControl_SaveConfig
********************************************************************************
*
* Summary:
*  Saves the control register value.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SDSPI_CSControl_SaveConfig(void) 
{
    SDSPI_CSControl_backup.controlState = SDSPI_CSControl_Control;
}


/*******************************************************************************
* Function Name: SDSPI_CSControl_RestoreConfig
********************************************************************************
*
* Summary:
*  Restores the control register value.
*
* Parameters:
*  None
*
* Return:
*  None
*
*
*******************************************************************************/
void SDSPI_CSControl_RestoreConfig(void) 
{
     SDSPI_CSControl_Control = SDSPI_CSControl_backup.controlState;
}


/*******************************************************************************
* Function Name: SDSPI_CSControl_Sleep
********************************************************************************
*
* Summary:
*  Prepares the component for entering the low power mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SDSPI_CSControl_Sleep(void) 
{
    SDSPI_CSControl_SaveConfig();
}


/*******************************************************************************
* Function Name: SDSPI_CSControl_Wakeup
********************************************************************************
*
* Summary:
*  Restores the component after waking up from the low power mode.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SDSPI_CSControl_Wakeup(void)  
{
    SDSPI_CSControl_RestoreConfig();
}

#endif /* End check for removal by optimization */


/* [] END OF FILE */
