/* ================================================================================
 *
 * DMASDSPI API.
 *
 * Copyright (c) 2016, KIICHIRO KOTAJIMA
 * All Rights Reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ================================================================================
*/
#include "project.h"
#include "cyfitter_sysint_cfg.h"
#include "SDSPI_api.h"
#include "SDSPI_fatfs.h"

#include "SDSPI_SPIMaster.h"
#include "SDSPI_CSControl.h"
#include "SDSPI_RX.h"
#include "SDSPI_TX.h"

#define TRANSFER_WAIT_RXDMAISR     1

#define BUFFER_SIZE    (64UL)
uint8_t bufferTx[BUFFER_SIZE];
uint8_t bufferRx[BUFFER_SIZE];

static volatile uint8_t nTransferFlags = 0;

static uint8_t bEnableDMA = 0;

static uint8_t dummy_data[1] = { 0xff };

#ifdef SDSPI_RX_DMA_IRQ__INTC_CORTEXM0P_PRIORITY

//CM0+ interrupt code

#endif
#ifdef SDSPI_RX_DMA_IRQ__INTC_CORTEXM4_PRIORITY
//CM4 interrupt code
void SDSPI_RX_DMA_ISR(){
	nTransferFlags &= ~TRANSFER_WAIT_RXDMAISR;
}
#endif

void SDSPI_Start(SDSPI_VOIDFXN FnSelSlowCk, SDSPI_VOIDFXN FnSelFastCk, SDSPI_PRNTFXN FnPrint)
{
	nTransferFlags = 0;
	SDSPI_SPIMaster_Start();
    //SDSPI_SPIMaster_Transfer(bufferTx, bufferRx, BUFFER_SIZE);
	SDSPI_SetDMAMode(0);
	
	SPI_FUNCTIONS SpiFuncs;
	SpiFuncs.SPRXSTREAM = SDSPI_ReceiveData;
	SpiFuncs.SPTXSTREAM = SDSPI_SendData;
	SpiFuncs.SPEXCHANGE = SDSPI_SwapByte;
	SpiFuncs.SPSELECTCARD = SDSPI_SelectCard;
	SpiFuncs.SPDESELECTCARD = SDSPI_DeselectCard;
	SpiFuncs.SPFASTCK = FnSelFastCk;
	SpiFuncs.SPSLOWCK = FnSelSlowCk;
	SpiFuncs.CARDSTATUS = SDSPI_CardStatus;
	SpiFuncs.PRINT = FnPrint;
	
	disk_attach_spifuncs(&SpiFuncs);
}

void SDSPI_Stop()
{
	SDSPI_DeselectCard();
	SDSPI_SetDMAMode(0);
	SDSPI_SPIMaster_Disable();
}

void SDSPI_SelectCard() {
	SDSPI_CSControl_Write(0);
}

void SDSPI_DeselectCard() {
	SDSPI_CSControl_Write(1);
}

uint8_t SDSPI_CardStatus() {
    return 0x02;
}

void SDSPI_SendData(const uint8_t* pBuffer, uint32_t nLength)
{
	cy_stc_dma_channel_config_t channelConfig;
    if((nLength > 4095)||(nLength < 1)){
		return; //this never happens - filesys transfers max. 512 bytes at a time
	}

	if(bEnableDMA) {
                
        //TX
        Cy_DMA_Descriptor_Init(&SDSPI_TX_Descriptor_1, &SDSPI_TX_Descriptor_1_config);
        Cy_DMA_Descriptor_SetSrcAddress(&SDSPI_TX_Descriptor_1, pBuffer);
        Cy_DMA_Descriptor_SetDstAddress(&SDSPI_TX_Descriptor_1, &SDSPI_SPIMaster_HW->TX_FIFO_WR);

        channelConfig.descriptor    = &SDSPI_TX_Descriptor_1;
        channelConfig.preemptable   = SDSPI_TX_PREEMPTABLE;
        channelConfig.priority      = SDSPI_TX_PRIORITY;
        channelConfig.enable        = 0u;

        Cy_DMA_Channel_Init(SDSPI_TX_DW__BLOCK_HW, SDSPI_TX_DW_CHANNEL, &channelConfig);
        Cy_DMA_Channel_SetInterruptMask(SDSPI_TX_DW__BLOCK_HW, SDSPI_TX_DW__CHANNEL_NUMBER, CY_DMA_INTR_MASK);
        Cy_DMA_Channel_Enable(SDSPI_TX_DW__BLOCK_HW, SDSPI_TX_DW__CHANNEL_NUMBER);
        
        
        //RX
        Cy_DMA_Descriptor_Init(&SDSPI_RX_Descriptor_1, &SDSPI_RX_Descriptor_1_config);
        Cy_DMA_Descriptor_SetSrcAddress(&SDSPI_RX_Descriptor_1, &SDSPI_SPIMaster_HW->RX_FIFO_RD);
        Cy_DMA_Descriptor_SetDstAddress(&SDSPI_RX_Descriptor_1, &dummy_data[0]);

        channelConfig.descriptor    = &SDSPI_RX_Descriptor_1;
        channelConfig.preemptable   = SDSPI_RX_PREEMPTABLE;
        channelConfig.priority      = SDSPI_RX_PRIORITY;
        channelConfig.enable        = 0u;

        Cy_DMA_Channel_Init(SDSPI_RX_DW__BLOCK_HW, SDSPI_RX_DW_CHANNEL, &channelConfig);
        Cy_DMA_Channel_SetInterruptMask(SDSPI_RX_DW__BLOCK_HW, SDSPI_RX_DW__CHANNEL_NUMBER, CY_DMA_INTR_MASK);
        Cy_DMA_Channel_Enable(SDSPI_RX_DW__BLOCK_HW, SDSPI_RX_DW__CHANNEL_NUMBER);
        
        Cy_DMA_Channel_ClearInterrupt(SDSPI_TX_DW__BLOCK_HW, SDSPI_TX_DW__CHANNEL_NUMBER);
        Cy_DMA_Channel_ClearInterrupt(SDSPI_RX_DW__BLOCK_HW, SDSPI_RX_DW__CHANNEL_NUMBER);   
        	
		nTransferFlags |= TRANSFER_WAIT_RXDMAISR;   //Set flag
		
		Cy_DMA_Enable(SDSPI_RX_DW__BLOCK_HW); //CyDmaChEnable(RXDMA_Channel, 1);
		Cy_DMA_Enable(SDSPI_TX_DW__BLOCK_HW); //CyDmaChEnable(TXDMA_Channel, 1);
		
		//CyDmaChSetRequest(TXDMA_Channel, CPU_REQ);  //trigger transfer

		// wait for transfer
		while(nTransferFlags & TRANSFER_WAIT_RXDMAISR);

		// stop DMA.
        Cy_DMA_Disable(SDSPI_TX_DW__BLOCK_HW);
        Cy_DMA_Disable(SDSPI_RX_DW__BLOCK_HW);
		return;
	}else{
		//Slow (non DMA) implementation...
		unsigned int i;
		for(i = 0; i < nLength; i++){
			SDSPI_SwapByte(pBuffer[i]);
		}
	}
}

//Send a block of NULL data (default 0xFF) and place received bytes into buffer.
void SDSPI_ReceiveData(uint8_t* pBuffer, uint32_t nLength)
{
	cy_stc_dma_channel_config_t channelConfig;
    if((nLength > 4095)||(nLength < 1)){
		return; //this never happens - filesys transfers max. 512 bytes at a time
	}

	if(bEnableDMA) {
		//  - single TD
		//  - dummy_data() => SPI_TXDATA       
        //TX
        Cy_DMA_Descriptor_Init(&SDSPI_TX_Descriptor_2, &SDSPI_TX_Descriptor_2_config);
        Cy_DMA_Descriptor_SetSrcAddress(&SDSPI_TX_Descriptor_2, &dummy_data[0]);
        Cy_DMA_Descriptor_SetDstAddress(&SDSPI_TX_Descriptor_2, &SDSPI_SPIMaster_HW->TX_FIFO_WR);
        
        channelConfig.descriptor    = &SDSPI_TX_Descriptor_2;
        channelConfig.preemptable   = SDSPI_TX_PREEMPTABLE;
        channelConfig.priority      = SDSPI_TX_PRIORITY;
        channelConfig.enable        = 0u;
        
        Cy_DMA_Channel_Init(SDSPI_TX_DW__BLOCK_HW, SDSPI_TX_DW_CHANNEL, &channelConfig);
        Cy_DMA_Channel_SetInterruptMask(SDSPI_TX_DW__BLOCK_HW, SDSPI_TX_DW__CHANNEL_NUMBER, CY_DMA_INTR_MASK);
        Cy_DMA_Channel_Enable(SDSPI_TX_DW__BLOCK_HW, SDSPI_TX_DW__CHANNEL_NUMBER);
		
		//  - single TD
		//  - SPI_RXDATA => pBuffer()
        //RX      
        Cy_DMA_Descriptor_Init(&SDSPI_RX_Descriptor_2, &SDSPI_RX_Descriptor_2_config);
        Cy_DMA_Descriptor_SetSrcAddress(&SDSPI_RX_Descriptor_2, &SDSPI_SPIMaster_HW->RX_FIFO_RD);
        Cy_DMA_Descriptor_SetDstAddress(&SDSPI_RX_Descriptor_2, pBuffer);
        
        channelConfig.descriptor    = &SDSPI_RX_Descriptor_2;
        channelConfig.preemptable   = SDSPI_RX_PREEMPTABLE;
        channelConfig.priority      = SDSPI_RX_PRIORITY;
        channelConfig.enable        = 0u;
        
        Cy_DMA_Channel_Init(SDSPI_RX_DW__BLOCK_HW, SDSPI_RX_DW_CHANNEL, &channelConfig);
        Cy_DMA_Channel_SetInterruptMask(SDSPI_RX_DW__BLOCK_HW, SDSPI_RX_DW__CHANNEL_NUMBER, CY_DMA_INTR_MASK);
        Cy_DMA_Channel_Enable(SDSPI_RX_DW__BLOCK_HW, SDSPI_RX_DW__CHANNEL_NUMBER);
        
		dummy_data[0] = 0xff;    
		nTransferFlags |= TRANSFER_WAIT_RXDMAISR;   //Set flag
    
        Cy_DMA_Channel_ClearInterrupt(SDSPI_TX_DW__BLOCK_HW, SDSPI_TX_DW__CHANNEL_NUMBER);
        Cy_DMA_Channel_ClearInterrupt(SDSPI_RX_DW__BLOCK_HW, SDSPI_RX_DW__CHANNEL_NUMBER);  

		Cy_DMA_Enable(SDSPI_RX_DW__BLOCK_HW);
		Cy_DMA_Enable(SDSPI_TX_DW__BLOCK_HW);

		//CyDmaChSetRequest(TXDMA_Channel, CPU_REQ);  //trigger transfer

		//wait until done.
		while(nTransferFlags & TRANSFER_WAIT_RXDMAISR);

		// stop DMA.
        Cy_DMA_Disable(SDSPI_TX_DW__BLOCK_HW);
        Cy_DMA_Disable(SDSPI_RX_DW__BLOCK_HW);
		return;
	} else {
		//Slow (non DMA) implementation...
		unsigned int i;
		for(i = 0; i < nLength; i++){
			pBuffer[i] = SDSPI_SwapByte(0xFF);
		}
	}
}

//Swap a single byte (send it, and receive RX byte / without DMA)
uint8_t SDSPI_SwapByte(uint8_t data)
{
    uint8_t readValue = 0;    
    SDSPI_SPIMaster_ClearRxFifo();
    SDSPI_SPIMaster_GetTxFifoStatus();//read to clear it 
    //write byte to TX
	SDSPI_SPIMaster_WriteArray(&data, 1);
	CyDelayUs(1);
	while(!(SDSPI_SPIMaster_IsTxComplete()));
    CyDelayUs(2);
    while(SDSPI_SPIMaster_GetNumInRxFifo() == 0);//wait data
    readValue= SDSPI_SPIMaster_Read();
    //SDSPI_SPIMaster_ReadArray(&readValue, 1);
	return readValue;
}

void SDSPI_SetDMAMode(unsigned char enable)
{
	if ((bEnableDMA && enable) || (!bEnableDMA && !enable))
		return;

	//make absolutely sure no transfers are in progress
	while(nTransferFlags);

	if (enable) {

        Cy_SysInt_Init(&SDSPI_RX_DMA_IRQ_cfg, SDSPI_RX_DMA_ISR);
        NVIC_EnableIRQ(SDSPI_RX_DMA_IRQ_cfg.intrSrc);

        Cy_DMA_Channel_ClearInterrupt(SDSPI_TX_DW__BLOCK_HW, SDSPI_TX_DW__CHANNEL_NUMBER);
        Cy_DMA_Channel_ClearInterrupt(SDSPI_RX_DW__BLOCK_HW, SDSPI_RX_DW__CHANNEL_NUMBER); 

		bEnableDMA = 1;
	} else {
		//Free Transfer
        NVIC_DisableIRQ(SDSPI_RX_DMA_IRQ_cfg.intrSrc);
		SDSPI_RX_ChannelDisable();
		SDSPI_TX_ChannelDisable();

		bEnableDMA = 0;
	}
}

/* [] END OF FILE */
