/* ================================================================================
 *
 * DMASDSPI API.
 *
 * Copyright (c) 2016, KIICHIRO KOTAJIMA
 * All Rights Reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * ================================================================================
*/
#if !defined(SDSPI_API_H)
#define SDSPI_API_H

#include <stdint.h>
    
typedef void (*SDSPI_VOIDFXN)(void);
typedef void (*SDSPI_PRNTFXN)(char*);
    
//Start component (preferred method - this sets up the filesystem too!)
void SDSPI_Start(SDSPI_VOIDFXN FnSelSlowCk, SDSPI_VOIDFXN FnSelFastCk, SDSPI_PRNTFXN FnPrint);

//Disable component, release DMAs, stop ISRs, etc
void SDSPI_Stop();

//Control the CS line
void SDSPI_SelectCard();
void SDSPI_DeselectCard();

//Return Card Status.
uint8_t SDSPI_CardStatus();

//Send a block of data and discard received bytes.
void SDSPI_SendData(const uint8_t* pBuffer, uint32_t nLength);

//Send a block of NULL data (default 0xFF) and place received bytes into buffer.
void SDSPI_ReceiveData(uint8_t* pBuffer, uint32_t nLength);

//Swap a single byte (send it, and receive RX byte)
uint8_t SDSPI_SwapByte(uint8_t data);

//Enable / Disable DMA transfers
void SDSPI_SetDMAMode(uint8_t bEnableDMA);


#endif
/* [] END OF FILE */
